#pragma once

#include <filesystem>

#include "../base/EntityInstanceBase.h"
#include "../workingmemory/EntityInstance.h"

#include "../workingmemory/entityInstance_conversions.h"
#include "../workingmemory/json_conversions.h"


namespace armarx::armem::d_ltm
{
    /**
     * @brief Data of a single entity instance.
     */
    class EntityInstance :
        public base::EntityInstanceBase<EntityInstance>
    {
        using Base = base::EntityInstanceBase<EntityInstance>;

    public:

        using Base::EntityInstanceBase;

        EntityInstance(const EntityInstance& other);
        EntityInstance(EntityInstance&& other) = default;
        EntityInstance& operator=(const EntityInstance& other) = default;
        EntityInstance& operator=(EntityInstance&& other) = default;


        /**
         * @brief Fill `*this` with the update's values.
         * @param update The update.
         * @param index The instances index.
         */
        virtual void update(const EntityUpdate& update, int index) override;

        virtual bool equalsDeep(const EntityInstance& other) const override;

        virtual EntityInstance copy() const override;

        // Conversion
        wm::EntityInstance convert(const aron::typenavigator::NavigatorPtr& expectedStructure) const;

        // Filesystem connection
        void reload(const std::shared_ptr<std::filesystem::path>&);
        void setTo(const wm::EntityInstance&);

    protected:
        virtual void _copySelf(EntityInstance& other) const override;

    private:
        std::filesystem::path _fullPath() const;
        std::filesystem::path _fullPath(const std::filesystem::path&) const;

    public:
        std::shared_ptr<std::filesystem::path> path;

    private:
        static const constexpr char* DATA_FILENAME = "data";
    };
}
