#include "TypeIO.h"

#include <iostream>
#include <fstream>
#include <filesystem>

#include <RobotAPI/libraries/aron/core/navigator/type/container/Object.h>
#include <RobotAPI/libraries/aron/core/io/typeIO/visitor/Visitor.h>
#include <RobotAPI/libraries/aron/core/io/typeIO/converter/Converter.h>
#include <RobotAPI/libraries/aron/core/io/typeIO/reader/nlohmannJSON/NlohmannJSONReader.h>
#include <RobotAPI/libraries/aron/core/io/typeIO/writer/navigator/NavigatorWriter.h>
#include <RobotAPI/libraries/aron/core/io/typeIO/writer/nlohmannJSON/NlohmannJSONWriter.h>


namespace armarx::armem::d_ltm
{

    aron::typenavigator::ObjectNavigatorPtr TypeIO::unwrapType(const aron::typenavigator::ObjectNavigatorPtr& type)
    {
        return aron::typenavigator::ObjectNavigator::DynamicCastAndCheck(type->getMemberType(TYPE_WRAPPER_DATA_FIELD));
    }

    aron::typenavigator::ObjectNavigatorPtr TypeIO::wrapType(const aron::typenavigator::ObjectNavigatorPtr& type)
    {
        aron::typenavigator::ObjectNavigatorPtr typeWrapped(new aron::typenavigator::ObjectNavigator());
        typeWrapped->setObjectName(type->getObjectName() + "__ltm_type_export");
        typeWrapped->addMemberType(TYPE_WRAPPER_DATA_FIELD, type);

        typeWrapped->addMemberType(TYPE_WRAPPER_TIME_STORED_FIELD, std::make_shared<aron::typenavigator::LongNavigator>());
        typeWrapped->addMemberType(TYPE_WRAPPER_TIME_CREATED_FIELD, std::make_shared<aron::typenavigator::LongNavigator>());
        typeWrapped->addMemberType(TYPE_WRAPPER_TIME_SENT_FIELD, std::make_shared<aron::typenavigator::LongNavigator>());
        typeWrapped->addMemberType(TYPE_WRAPPER_TIME_ARRIVED_FIELD, std::make_shared<aron::typenavigator::LongNavigator>());
        typeWrapped->addMemberType(TYPE_WRAPPER_CONFIDENCE_FIELD, std::make_shared<aron::typenavigator::DoubleNavigator>());

        return typeWrapped;
    }

    aron::typenavigator::ObjectNavigatorPtr TypeIO::readAronType(const std::filesystem::__cxx11::path& filepath)
    {
        if (std::filesystem::is_regular_file(filepath))
        {
            if (filepath.filename() == (std::string(TYPE_FILENAME) + ".json"))
            {
                std::ifstream ifs(filepath);
                std::string file_content((std::istreambuf_iterator<char>(ifs)), (std::istreambuf_iterator<char>()));

                aron::typeIO::reader::NlohmannJSONReader typeReader(file_content);
                aron::typeIO::writer::NavigatorWriter navWriter;
                aron::typeIO::Converter::ReadAndConvert(typeReader, navWriter);
                return aron::typenavigator::ObjectNavigator::DynamicCastAndCheck(navWriter.getResult());
            }
        }
        return nullptr;
    }

    void TypeIO::writeAronType(const aron::typenavigator::ObjectNavigatorPtr& type, const std::filesystem::__cxx11::path& filepath)
    {
        if (type)
        {
            std::ofstream ofs(filepath);

            aron::typeIO::writer::NlohmannJSONWriter typeWriter;
            aron::typeIO::Visitor::VisitAndSetup(typeWriter, type);
            std::string new_file_full_content = typeWriter.getResult().dump(2);

            ofs << new_file_full_content;
        }
    }


}
