#pragma once

// BaseClass
#include "../base/MemoryBase.h"

// CoreSegment
#include "CoreSegment.h"

// WM
#include "../workingmemory/Memory.h"

// Properties
#include <ArmarXCore/core/application/properties/PluginAll.h>
#include <ArmarXCore/core/application/properties/PropertyDefinitionContainer.h>

namespace armarx::armem::ltm
{
    /**
     * @brief Data of a memory consisting of multiple core segments.
     */
    class Memory :
        public base::MemoryBase<CoreSegment, Memory>
    {
        using Base = base::MemoryBase<CoreSegment, Memory>;

    public:

        using Base::MemoryBase;
        using Base::operator=;

        struct TransferSettings
        {
            bool enabled = false;
        };

        struct AlwaysTransferSettings : public TransferSettings
        {

        };

        struct PeriodicTransferSettings : public TransferSettings
        {
            bool deleteFromWMOnTransfer = false;
            int frequencyHz = 1;
        };

        struct OnFullTransferSettings : public TransferSettings
        {
            enum class Mode
            {
                TRANSFER_LATEST,
                TRANSFER_LEAST_USED
            };

            Mode mode;
        };


        Memory(const Memory& other);
        Memory(Memory&& other);
        Memory& operator=(const Memory& other);
        Memory& operator=(Memory&& other);

        // PropertyDefinitions related to LTM
        void defineProperties(armarx::PropertyDefinitionsPtr defs, const std::string& prefix = "");

        // Conversion
        wm::Memory convert() const;

        // MongoDB connection
        void reload();
        void append(const wm::Memory&);

    protected:
        virtual void _copySelf(Memory& other) const override
        {
            Base::_copySelf(other);
            other.dbsettings = dbsettings;
        }

        virtual void _copySelfEmpty(Memory& other) const override
        {
            Base::_copySelfEmpty(other);
            other.dbsettings = dbsettings;
        }

    private:
        bool checkConnection() const;

        void _append(const wm::Memory&);

    public:
        MongoDBConnectionManager::MongoDBSettings dbsettings;

        AlwaysTransferSettings alwaysTransferSettings;
        PeriodicTransferSettings periodicTransferSettings;
        OnFullTransferSettings onFullTransferSettings;

    private:
        bool reloaded = false;
        mutable std::mutex mongoDBMutex;

        std::vector<wm::Memory> toAppendQueue;
    };
}
