#pragma once

#include <RobotAPI/libraries/armem/core/base/MemoryBase.h>

#include "CoreSegment.h"
#include "detail/CopyWithoutData.h"


namespace armarx::armem::wm
{

    /**
     * @brief Data of a memory consisting of multiple core segments.
     */
    class Memory :
        public base::MemoryBase<CoreSegment, Memory>,
        public detail::CopyWithoutData<Memory>
    {
        using Base = base::MemoryBase<CoreSegment, Memory>;

    public:

        using Base::MemoryBase;

        Memory(const Memory& other) = default;
        Memory(Memory&& other) = default;
        Memory& operator=(const Memory& other) = default;
        Memory& operator=(Memory&& other) = default;

        /**
         * @brief Perform the commit, locking the core segments.
         *
         * Groups the commits by core segment, and updates each core segment
         * in a batch, locking the core segment.
         */
        std::vector<Base::UpdateResult> updateLocking(const Commit& commit);

        /**
         * @brief Update the memory, locking the updated core segment.
         */
        Base::UpdateResult updateLocking(const EntityUpdate& update);


        /**
         * @brief Convert the content of this memory into a commit
         * @return The resulting commit
         */
        Commit toCommit() const;


    protected:

        virtual void _copySelfWithoutData(Memory& other) const override;

    };
}
