#pragma once

#include <ArmarXCore/core/logging/Logging.h>
#include <RobotAPI/interface/armem/query.h>


namespace armarx::armem::base::query_proc
{

    /**
     * @brief Base class for memory query processors.
     */
    template <class DataT, class QueryT>
    class BaseQueryProcessorBase
    {
    public:
        using QueryPtrT = ::IceInternal::Handle<QueryT>;
        using QuerySeqT = std::vector<QueryPtrT>;

    public:

        virtual ~BaseQueryProcessorBase() = default;


        DataT process(const QueryT& query, const DataT& data, const query::data::QueryTargets& executeIf = {}) const
        {
            DataT result = data.copyEmpty();

            for (const auto queryTarget : query.targets)
            {
                if (std::find(executeIf.begin(), executeIf.end(), queryTarget) != executeIf.end())
                {
                    this->process(result, query, data);
                    break;
                }
            }
            return result;
        }

        DataT process(const QueryPtrT& query, const DataT& data, const query::data::QueryTargets& executeIf = {}) const
        {
            return this->process(*query, *data, executeIf);
        }

        DataT process(const QuerySeqT& queries, const DataT& data, const query::data::QueryTargets& executeIf = {}) const
        {
            DataT result = data.copyEmpty();
            this->process(result, queries, data, executeIf);
            return result;
        }

        void process(DataT& result, const QuerySeqT& queries, const DataT& data, const query::data::QueryTargets& executeIf = {}) const
        {
            if (queries.empty())
            {
                ARMARX_DEBUG << "There are no queries to process.";
                return;
            }

            if (executeIf.empty())
            {
                ARMARX_DEBUG << "Could not execute query. ExecuteIf s empty.";
                return;
            }

            for (const auto& query : queries)
            {
                if (query->targets.empty())
                {
                    ARMARX_DEBUG << "The targets of a query are empty";
                    continue;
                }

                for (const auto queryTarget : query->targets)
                {
                    if (std::find(executeIf.begin(), executeIf.end(), queryTarget) != executeIf.end())
                    {
                        this->process(result, *query, data);
                        break;
                    }
                    else
                    {
                        ARMARX_DEBUG << "The query target " << queryTarget << " was not found in executeIf: " << executeIf;
                    }
                }
            }
        }

        virtual void process(DataT& result, const QueryT& query, const DataT& data) const = 0;
    };
}
