#include "PeriodicUpdateWidget.h"

#include <QCheckBox>
#include <QDoubleSpinBox>
#include <QPushButton>
#include <QTimer>
#include <QHBoxLayout>

#include <cmath>


namespace armarx::armem::gui
{

    PeriodicUpdateWidget::PeriodicUpdateWidget(double frequency, double maxFrequency)
    {
        setSizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        QLayout* layout = new QHBoxLayout();
        this->setLayout(layout);

        const int margin = 0;
        layout->setContentsMargins(margin, margin, margin, margin);

        _updateButton = new QPushButton("Update", this);
        _autoCheckBox = new QCheckBox("Auto Update", this);
        _frequencySpinBox = new QDoubleSpinBox(this);
        _frequencySpinBox->setValue(frequency);
        _frequencySpinBox->setMinimum(0);
        _frequencySpinBox->setMaximum(maxFrequency);
        _frequencySpinBox->setSingleStep(1.0);
        _frequencySpinBox->setSuffix(" Hz");


        layout->addWidget(_updateButton);
        layout->addWidget(_autoCheckBox);
        layout->addWidget(_frequencySpinBox);

        _timer = new QTimer(this);
        _updateTimerFrequency();
        _frequencySpinBox->setEnabled(_autoCheckBox->isChecked());


        // Private connections.
        connect(_autoCheckBox, &QCheckBox::toggled, this, &This::_toggleAutoUpdates);
        connect(_frequencySpinBox, &QDoubleSpinBox::editingFinished, this, &This::_updateTimerFrequency);

        // Public connections.
        connect(_updateButton, &QPushButton::pressed, this, &This::update);
        connect(_timer, &QTimer::timeout, this, &This::update);
    }

    QPushButton* PeriodicUpdateWidget::updateButton()
    {
        return _updateButton;
    }

    void PeriodicUpdateWidget::startTimerIfEnabled()
    {
        if (_autoCheckBox->isChecked())
        {
            _timer->start();
        }
        else
        {
            _timer->stop();
        }
    }

    void PeriodicUpdateWidget::stopTimer()
    {
        if (_timer)
        {
            _timer->stop();
        }
    }

    void PeriodicUpdateWidget::_updateTimerFrequency()
    {
        _timer->setInterval(static_cast<int>(std::round(1000 / _frequencySpinBox->value())));
    }

    void PeriodicUpdateWidget::_toggleAutoUpdates(bool enabled)
    {
        _frequencySpinBox->setEnabled(enabled);
        if (enabled)
        {
            _timer->start();
        }
        else
        {
            _timer->stop();
        }
    }

    QCheckBox* PeriodicUpdateWidget::autoCheckBox()
    {
        return _autoCheckBox;
    }

    QDoubleSpinBox* PeriodicUpdateWidget::frequencySpinBox()
    {
        return _frequencySpinBox;
    }

    QTimer* PeriodicUpdateWidget::timer()
    {
        return _timer;
    }

}

