// STD / STL
#include <iostream>
#include <fstream>
#include <sstream>

// BaseClass
#include "Segment.h"

// ArmarX
#include "MotionConverter.h"
#include <RobotAPI/libraries/PriorKnowledge/motions/MotionFinder.h>

namespace armarx::armem::server::motions::mdb
{
    Segment::Segment(armem::server::MemoryToIceAdapter& memoryToIceAdapter) :
        Base(memoryToIceAdapter)
    {
    }

    void Segment::defineProperties(armarx::PropertyDefinitionsPtr defs, const std::string& prefix)
    {
        Base::defineProperties(defs, prefix);

        defs->optional(p.motionsPackage, prefix + "MotionsPackage", "Name of the prior knowledge package to load from.");
        defs->optional(p.loadFromMotionsPackage, prefix + "LoadFromMotionsPackage", "If true, load the motions from the motions package on startup.");
    }

    void Segment::onInit()
    {
        Base::onInit();

        if (p.loadFromMotionsPackage)
        {
            loadByMotionFinder(p.motionsPackage);
        }
    }

    void Segment::onConnect()
    {

    }

    int Segment::loadByMotionFinder(const std::string& packageName)
    {
        priorknowledge::motions::MotionFinder motionFinder(packageName, "motions/");
        int loadedMotions = 0;

        std::cout << "Load Motions from package" << std::endl;
        {
            // Load MDB Motions
            auto allMotions = motionFinder.findAll("mdb");
            for (const auto& motionFinderInfo : allMotions)
            {
                auto pathToInfoJson = motionFinderInfo.getFullPath() / motionFinderInfo.getID() / (motionFinderInfo.getID() + ".json");
                if (auto op = conversion::createFromFile(pathToInfoJson); op.has_value())
                {
                    std::stringstream ss;
                    ss << "Found valid instance at: " << pathToInfoJson << ". The motionID is: ";
                    for (const auto& [key, attachedFiles] : op->attachedFiles)
                    {
                        for (const auto& attachedFile : attachedFiles)
                        {
                            ss << "\n" << attachedFile.fileName << " (" << key << ")";
                        }
                    }
                    ARMARX_INFO << ss.str();
                    auto& entity = this->segment->addEntity(std::to_string(op->id));
                    auto& snapshot = entity.addSnapshot(op->createdDate);

                    armem::wm::EntityInstance& instance = snapshot.addInstance();
                    instance.metadata().timeCreated = op->createdDate;
                    instance.metadata().timeSent = IceUtil::Time::now();
                    instance.metadata().timeArrived = IceUtil::Time::now();
                    instance.metadata().confidence = 1.0;
                    instance.setData(op->toAron());
                }
                else
                {
                    ARMARX_WARNING << "Found an invalid path to a motion file: " << pathToInfoJson;
                }
            }
            IceUtil::Time::now();

            loadedMotions += allMotions.size();
        }

        return loadedMotions;
    }
}
