/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <string>
#include <unordered_map>

// ArmarX
#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/core/navigator/Navigator.h>
#include <RobotAPI/libraries/aron/core/Path.h>
#include <RobotAPI/libraries/aron/core/navigator/type/Navigator.h>

namespace armarx::aron::datanavigator
{
    class NavigatorFactory;
    typedef std::shared_ptr<NavigatorFactory> NavigatorFactoryPtr;

    class Navigator;
    typedef std::shared_ptr<Navigator> NavigatorPtr;

    class Navigator :
        virtual public aron::Navigator<data::Descriptor, data::AronData>
    {
    public:
        using PointerType = NavigatorPtr;

    public:
        // constructors
        Navigator() = default;

        // operators
        virtual bool operator==(const Navigator& other) const = 0;
        virtual bool operator==(const NavigatorPtr& other) const = 0;

        // static methods
        static NavigatorPtr FromAronData(const data::AronDataPtr&, const Path& = Path());
        static std::vector<NavigatorPtr> FromAronData(const std::vector<data::AronDataPtr>&, const Path& = Path());
        static std::vector<data::AronDataPtr> ToAronData(const std::vector<NavigatorPtr>&);

        template<typename NavigatorT>
        static typename NavigatorT::PointerType DynamicCast(const NavigatorPtr& n)
        {
            return NavigatorT::DynamicCast(n);
        }

        template<typename NavigatorT>
        static typename NavigatorT::PointerType DynamicCast(const Navigator& n)
        {
            return NavigatorT::DynamicCast(n);
        }

        template<typename NavigatorT>
        static typename NavigatorT::PointerType DynamicCastAndCheck(const NavigatorPtr& n)
        {
            return NavigatorT::DynamicCastAndCheck(n);
        }

        // virtual definitions
        virtual std::vector<NavigatorPtr> getChildren() const = 0;
        virtual size_t childrenSize() const = 0;

        virtual typenavigator::NavigatorPtr recalculateType() const = 0;
        virtual bool fullfillsType(const typenavigator::NavigatorPtr&) const = 0;

        virtual NavigatorPtr navigateAbsolute(const Path& path) const;
        virtual NavigatorPtr navigateRelative(const Path& path) const;

        // public member functions
        data::AronDataPtr toAronDataPtr() const
        {
            return toAronPtr();
        }

    private:
        static const NavigatorFactoryPtr FACTORY;
    };
}
